/* scratchpad - Clipboard-based prototype of the Netscape Scratchpad system
 * Copyright (C) 1998-2000 by Patrick Reynolds <reynolds@cs.duke.edu>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include <string.h>

#include <gtk/gtkclist.h>
#include <gtk/gtkdnd.h>
#include <gtk/gtksignal.h>

enum { TARGET_STRING, TARGET_URL };
static GtkTargetEntry target_table[] = {
  { "STRING",        0, TARGET_STRING },
  { "text/plain",    0, TARGET_STRING },
  { "text/uri-list", 0, TARGET_URL }
};
static int n_targets = sizeof(target_table) / sizeof(target_table[0]);

extern char *url_prefix[];

static void drag_data_received(GtkWidget *widget, GdkDragContext *context,
    int x, int y, GtkSelectionData *data, int info, int time);

void init_dnd(GtkWidget *w) {
  g_return_if_fail(w != NULL);
  g_return_if_fail(GTK_IS_WIDGET(w));
  gtk_drag_dest_set(w, GTK_DEST_DEFAULT_ALL, target_table, n_targets-1,
    GDK_ACTION_COPY|GDK_ACTION_MOVE);
  gtk_signal_connect(GTK_OBJECT(w), "drag_data_received",
    GTK_SIGNAL_FUNC(drag_data_received), NULL);
}

static void drag_data_received(GtkWidget *widget, GdkDragContext *context,
    int x, int y, GtkSelectionData *data, int info, int time) {
  g_return_if_fail(widget != NULL);
  g_return_if_fail(GTK_IS_CLIST(widget));
  g_return_if_fail(context != NULL);
  g_return_if_fail(data != NULL);
  if ((data->length >= 0) && (data->format == 8)) { /*!! 8? */
    int i, flag=0;
    char *url = (char*)data->data;
    for (i=0; url_prefix[i]; i++)
      if (!strncasecmp(url, url_prefix[i], strlen(url_prefix[i]))) {
        flag = 1;
        break;
      }
    if (flag) {
      gtk_clist_append(GTK_CLIST(widget), &url);
      gtk_drag_finish(context, TRUE, FALSE, time);
      return;
    }
  }
  gtk_drag_finish(context, FALSE, FALSE, time);
}
